;/****************************************************************
; Interrupt subroutines and RF drivers
; Adjusted the send_RF timing. The 26us sleep time is restared after every word transfer to
; give the receiver more time to get ready before the start bit transition is initiated. 
;****************************************************************/
#include <msp430x44x.h>
.iar
#include  "std_f43x.asm"									  
#define rf_X_buf_full	0x4
#define uart_R_buf_full	0x8
#define LOCAL_BUTTON_PUSHED	0x8
#define rf_R_buf_full	0x10
#define RF_preamble_wait 0x20
#define RF_timedout_flag 0x80
#define F_change_flag 0x100


HW_FLOW_CONTROL   EQU     0200h
TEST_FLAG	EQU	01     ;P5.0 is the test flag
;PREAMBLE_LENGTH  EQU   0154     ;09AH,
PREAMBLE_LENGTH  EQU 400     ; mgs was 0400 hence octal
;PREAMBLE_LENGTH  EQU 100     ;
;PREAMBLE_LENGTH  EQU 0200     ;
PREAMBLE2  EQU PREAMBLE_LENGTH*2+8
MaxEdgeCount EQU 628          ; used to be 0134h before Pre-amble got adjusted; mgs used to be 0628 hence octal
RED     EQU 08h
ORN     EQU 04h
YEL     EQU 02h
GRN     EQU 01h
ORN_TH  EQU 0313h
YEL_TH  EQU 0617h
GRN_TH  EQU 0A41h

T10mS      EQU   24576            ;mgs octal
T1mS       EQU   2457          ;1mS using timer A;mgs octal
T12mS      EQU   4914          ;2mS using timer A;mgs octal
T13mS      EQU   7371          ;3mS;mgs octal
T10mSB     EQU   3072           ;10mS using timer B 
;T1mSB     EQU   615            ;1mS using timer B 
T1mSB     EQU   1230            ;1mS=307 using timer B 
;ShortLimit   EQU   56           ;Both edge 
;LongLimit    EQU    20

ShortLimit   EQU   120           ;Negative edge
LongLimit    EQU    16            ;48-55 us clock cycle


MAXPREAMBLE2START_GAP EQU  T12mS  
#define   RSTAT			 R8				  ; status of the reception
#define   wait_r		 R9				  ; counter register for all waiting loops ***** 
#define   counter		 R10			  ; universal counter
;	NAME	radio(16)
;	RSEG	CODE(1)
;	COMMON	INTVEC(1)
 ;       .EXTERNAL  _time_out_count
 
        .EXTERNAL  _Btime
        .EXTERNAL  _Btime1_5 
        .EXTERNAL  _ComFlag
        .EXTERNAL  _trf6901
;	.EXTERNAL	_rx_buff
;        .EXTERNAL  _tx_buff
;	.EXTERNAL	_tx_RS232_b
 ;      .EXTERNAL  _flash_ram
        .EXTERNAL  _Set_DCO
	.GLOBAL	Timer_A1
        .GLOBAL  WDT
	.GLOBAL	PORT2_INT
	.GLOBAL	_receive_RF
	.GLOBAL	_send_RF
        .GLOBAL  hold_radio
        .GLOBAL  release_radio
;	.EXTERNAL	CL430_1_23_L08
;	RSEG	CODE
    .pseg radiocode,abs=0x3122
Timer_A1:
		  ADD	  &TAIV,PC									
		  RETI
		  JMP	  CC1_INT									; RS232 reception -> falling edge of the 
															; startbit
		  JMP	  CC2_INT									; RF reception -> every edge of 
															; the rx-signal
		  RETI
		  RETI
back		  RETI

;**************************** Timer A0 Interrupt ****************************************************
; last change:10/802
;
; used for timeout between Preamble and Start
;
;*******************************************************************************************************
Timer_A0:
		  BIC	  #CCIE,&CCTL0								; disable CCTL2
                  BIS     #RF_timedout_flag,&_ComFlag
                  RETI

;**************************** WDT Interrupt Routine ****************************************************
; used for periodical wakeup during the listen phase
;
;*******************************************************************************************************

WDT
		  BIC	  #CCIE,&CCTL2								; disable CCTL2
wdt_ret
		  BIC	  #CPUOFF,0(SP)								; reactivate CPU
		  RETI																					  

;****************************PORT2 Interrupt Routine
;last change 4/9/01	. This is the wake up call hooked up to the UART
;***************************************************

PORT2_INT
 	 	 BIC.B	#BIT7,P2IFG									;
  		 BIC.B	#BIT7,P2IE									;
		 BIC	#CPUOFF,0(SP)								; reactivate CPU
		 RETI	

;****************************** Capture Compare 2 Register *********************************************
; last change: October 19th 1999
;
; used for RF-Reception
;
;*******************************************************************************************************

CC2_INT											; used by biph_rx 
  
		  MOV	  RRFTAB(R8),PC							; conditional jump depends on RSTAT
											; RSTAT = 0, detecting the Trianingsequence
											; RSTAT = 1, Trainingsquence detected, waiting 
											; for the Start Bit
											; RSTAT = 2, Start Bit detected, Data Reception
RRFTAB	          DW	  RSTAT00							 
		  DW	  RSTAT01
		  DW	  RSTAT10
		  DW	  RSTAT11
		  
RSTAT00
		  BIT	  #LOCAL_BUTTON_PUSHED+RF_timedout_flag,&_ComFlag 	; UART receive char is valid
		  JNZ	  kill_receive                  
		  MOV	  &CCR2,R14			; save Reference Capture value
		  MOV	  R14,R15			; copy Timer_A value 
		  SUB	  R13,R15			; subtract the current Timer_A value from the
															; old one -> Bitwidth in cycles in res_r
		  MOV	  R14,R13			; current value now -> old value later

test_res_r00
		  ;SUB	  #038h,R15			; subtract the average value from the 
		  ;CMP	  #010h,R15		        ;4/18	is the detected signal 51-63 cycles long?
		  SUB	  #ShortLimit,R15			; subtract the average value from the 
		  CMP	  #LongLimit,R15		        ;4/18	is the detected signal 20-34uS long?
		  JHS	  no_valid_pulse
		  INCD	  R8				; first valid pulse detected
		  INC	  wake_up_counter		; count this valid pulse
		  BIS	  #CCIE,&CCTL2			; re-enable CCR2 interrupt
		  JMP	  go_back

no_valid_pulse
		  CLR	  R8									; no the signal doesn't fit the wakeup sequence
		  CLR	  wake_up_counter							; reset the wake_up_counter, received an 
															; invalid pulse 
		  JMP	  go_back

RSTAT01
		  BIT	  #LOCAL_BUTTON_PUSHED+RF_timedout_flag,&_ComFlag 	; UART receive char is valid
		  JNZ	  kill_receive                  
		  MOV	  &CCR2,R14							; save Reference Capture value
		  MOV	  R14,R15							; copy Timer_A value 
		  SUB	  R13,R15							; subtract the current Timer_A value from the
															; old one -> Bitwidth in cycles in res_r
		  MOV	  R14,R13						; current value now -> old value later

test_res_r01
		  ;SUB	  #038h,R15						; subtract the average value from the 
		  ;CMP	  #010h,R15	;4/18					; is the detected signal 51-63 cycles long?
		  SUB	  #ShortLimit,R15			; subtract the average value from the 
		  CMP	  #LongLimit,R15		        ;4/18	is the detected signal 20-34uS long?
		  JHS	  no_valid_pulse
		  INC	  wake_up_counter					; next valid pulse
		  JMP	  go_back
		  
RSTAT10
		  MOV	  &CCR2,R14							; save Reference Capture value
		  MOV	  R14,R15							; copy Timer_A value 
		  SUB	  R13,R15							; subtract the current Timer_A value from 
															; the old one -> Bitwidth in cycles in res_r
		  MOV	  R14,R13				; current value now -> old value later

test_res_r10                                                    ;at 38.4k 73-81us window
		   SUB	   #0AFh,R15				; subtract the average value from the 
;;;;;		   SUB	   #0B0h,R15				; *******
;		   SUB	  #0B5h,R15	                	; subtract the average value from the 
								; measured value
;		   SUB	   #0BAh,R15 
;		   SUB	   #0BFh,R15
;		   SUB	   #0C5h,R15
;		   SUB	   #0CFh,R15
		  ;CMP	  #027h,R15						; is the detected signal x cycles long?

		  ;CMP	  #014h,R15	                	; is the detected signal x cycles long? normal to 5/1/03
		  CMP	  #42,R15	                	; is the detected signal x cycles long? This is for asymmetric stuff;mgs used to be 042 hence octal
		  JGE	  invalid_bit					; restart detection, this is not a valid sequence
		  JHS	  no_start_int_enable
		  INCD	  R8							; go to RSTATE 2, Data Reception, Start Bit 
													; detected
		  BIC	  #CCIE,&CCTL2				; disable CCR2 interrupt 
		  BIC	  #022h,&TACTL					; stop Timer_A and disable interrupt

no_start_bit
		  INC	  wake_up_counter			; count the pulses of the trainings sequence,
		  CMP	  #MaxEdgeCount,wake_up_counter		; compare the value of the counter with the 
		  JGE	  invalid_bit				;		  
		  CMP	  #06h,R8				; has the start bit been detected?	  
		  JNE	  go_back				; if not, return right away
		  MOV	  #WDTPW+WDTTMSEL+WDTCNTCL+03h,&WDTCTL		; if start detected, WDT set to timeout every 52uS
		  JMP	  go_back						;get ready for data collection

no_start_int_enable
		  JMP 	  no_start_bit

invalid_bit
		  CLR	  R8						; restart the detection, this is not a valid
		  CLR	  wake_up_counter				; initialize the wake_up_counter

go_back
		  BIC	  #CPUOFF,0(SP)					; wake up!
		  RETI
;RF data bit handler with UART type resynchronization


RSTAT11
		  BIS.B	  #TEST_FLAG,&P5OUT				;mark the start of start bit
		  BIT	  #CAP,&CCTL2					; if in capture mode, just found start bit 
	          JNZ	  R_start_edge_detected				; due to negative edge

		  BIC.B	 #rx,&P2SEL					;make it into GPIO
		  BIT.B	 #rx,&P2IN					;read IObit
		  RLC	  data_r					; push carry into the data register
		  BIS.B	 #rx,&P2SEL					;go back to module

                  ADD	  &_Btime,&CCR2
		  INC	  bits_r
		  CMP	  #010h,bits_r					; receive 16 bits in row
		  JNE	  end_RSTAT11_INT 				;get all bits
		  BIC	  #CCIE,&CCTL2					;no more int
		  JMP	  end_RSTAT11_INT

R_start_edge_detected
		  BIC	  #CAP,&CCTL2					;go to compare mode
		  ADD	  &_Btime1_5,&CCR2 		  		;get ready to receive data
		  CLR	  bits_r

end_RSTAT11_INT	  
									; before the interrupt request, but start
		  BIC.B	  #TEST_FLAG,&P5OUT				; mark the start of start bit
		  BIC	  #CPUOFF,0(SP)					; wake up!
									; subroutine
		  RETI
kill_receive
		  BIC	  #CCIE,&CCTL2				; disable CCR2 interrupt
		  BIC	  #022h,&TACTL				; stop Timer_A and disable interrupt	 
		  BIC	  #CPUOFF,0(SP)				; wake up!
		  RETI
                  

;****************************** Capture Compare 1 Register *********************************************
;Place holder for the Uart receive interrupt driver using CC1_INT
;Obsolete
;*******************************************************************************************************

CC1_INT
		  BIC	  #CCIE,&CCTL1								; disable CCR1 interrupt
															; subroutine
		  RETI

;*********************************** send_RF ***********************************************************
; last change: 12/12/00
;
; purpose:  sends data through RF channel 
; the calling function must set: rf_X_buf_full	EQU	04h
;                                load TempReg withsize N(total # of bytes)
;                                N must be even 
;The routine clears rf_X_buf_full
;The routine resynchs after every word
;*******************************************************************************************************

_send_RF

data_to_transmit
		  BIT	  #rf_X_buf_full,&_ComFlag						; is there any data to send via TRF6900?
		  JZ	  skip_send_RF								; no nothing to send
		  DINT
                  MOV      R12,R10
                  MOV      R12,R6 
                  ADD      R13,R10    ;mgs modif!!!
                  ;CALL    program_TX_mode_0                ; new for TRF6901
		  BIC.B	  #mode,&P3OUT	                    ; Mode=0  -> Send mode
		  BIC.B	  #tx,&P2OUT	                    ; TXDATA(P2.2) is reset
		  BIS.B	  #stdb_trf6901,&P3OUT		    ; TRF6900 active, STANDBY(P3.5) is high
		  MOV	  #WDTPW+WDTHOLD,&WDTCTL	    ; Password, Watchdog Timer hold 
                  DECD    R10
                  DECD    R6                                ;rf_buff_size-2=txrf data
		;  MOV	  0(R10),data_r		  	    ;  first word to the send register							  
            MOV.w @R10,data_r                    ;mgs modif
		  MOV	  #WDTPW+WDTTMSEL+WDTCNTCL+03h,&WDTCTL		; Reset, Timer Mode, Password, every 26,04sec			 
		  BIS.B	  #01h,&IE1					; enable Watchdog Timer interrupt

send_RF_training_sequence						; the entire length ca. 4ms, 154 pulses

		   MOV	  #PREAMBLE_LENGTH, tr_counter			; =154 drop to 40? initialize the training sequence counter
                  
send_RF_toggle
		  BIS	  #CPUOFF+GIE,SR				; CPU off
;------------------ Start of the trainings sequence ----------------------------------------------------
		  XOR.B	  #tx,&P2OUT					; toggle TXDATA(P2.2)
		  DEC	  tr_counter					; decrement counter for the training sequence
        	  JNZ	  send_RF_toggle				; length of the loop exactly 26,04sec
;;;;;;;;;*****
;                  JMP     send_RF_toggle
;;;;;;;;;;;;;************************************************************test only
send_RF_long_bit
		  BIS	  #CPUOFF+GIE,SR				; CPU off
;------------------ Start of the start bit -------------------------------------------------------------
		  BIS.B	  #tx,&P2OUT					; start of the long start-bit 78,12sec
		  BIS	  #CPUOFF+GIE,SR				; CPU off
		  BIC	  #04h,&_ComFlag				        ; the RS232 buffer is ready for reception 
		  BIS	  #CPUOFF+GIE,SR				; CPU off
		  BIS	  #CPUOFF+GIE,SR				; CPU off
;----------------- End of the start bit ----------------------------------------------------------------
start_bit_low
		  BIC.B	  #tx,&P2OUT					; reset TXDATA(P2.2)
		  BIS	  #CPUOFF+GIE,SR				; CPU off

send_RF_data
		  MOV	  #010h,bits_r					; init bitcounter, transmit first 16 bits		  

send_RF_bit_test
		  RLC	  data_r					; push the next data bit to carry
		  JC	  send_RF_high								

send_RF_low
		  BIC.B	  #tx,&P2OUT					; reset TXDATA(P2.2)
		  BIS	  #CPUOFF+GIE,SR				; CPU off
;-------------------------------- Start of the Databit -------------------------------------------------

send_RF_next_word
		  DEC	  bits_r					; decrement bit counter
		  JNZ	  send_RF_bit_test
		  DECD	  R10						; decrement word counter
                  DECD    R6
		  JN	  send_RF_LED3_on				; all data has been transmitted
           ;       MOV     0(r10),data_r    ;mgs modif
           MOV.W @R10,data_r
		  BIS.B	  #tx,&P2OUT					; add the logic high stop bit TXDATA(P2.2)
;******************************The following for  generic timing adjustment****************************
                  NOP
                  NOP
		  MOV	  #WDTPW+WDTTMSEL+WDTCNTCL+03h,&WDTCTL		 ; Reset, Timer Mode, Password, every 26,04sec			 
;******************************11-05-01****************************************************************
		  BIS	  #CPUOFF+GIE,SR				 ; CPU off
;******the following added for the new RF data format
		  JMP	  start_bit_low					 ; get next word by sending active low start bit first

send_RF_high
		  BIS.B	  #tx,&P2OUT					 ; set TXDATA(P2.2)
		  BIS	  #CPUOFF+GIE,SR				 ; CPU off
;-------------------------------- Start of the Databit -------------------------------------------------
		  JMP	  send_RF_next_word

send_RF_LED3_on
		  BIS	  #CPUOFF+GIE,SR				 ; CPU off-accomodate TRF6901 timing
		  EINT

end_send_RF 

		  MOV	  #WDTPW+WDTHOLD,&WDTCTL			; stop Watchdog Timer 
		  BIC.B	  #tx,&P2OUT							  
		  BIC.B	  #stdb_trf6901,&P3OUT				; clear STDBY(P3.5), TRF6901 standby mode
skip_send_RF
                  ;BIC.B   #080h,&P3OUT
		  RET

;********************************* receive_RF **********************************************************
; last change: 12/18/00
;
; main routine for code reception
; features:
; - calling function clears RF_preamble_wait
; - calling function loads the number of bytes to be transfered;flash_ram.rf_r_buff_size
; - rf_R_buf_full is cleared by parse_RF routine
; - once the RF chip initialized it waits at least 1 ms for it to settle. This in absence of lock det.
; - the function sets up and then waits for pre-amble detection 
; - receive new data format to allow for more than 4 bytes
; - received data is loaded into rx_buff[0]
; 
;*******************************************************************************************************

_receive_RF
                  MOV     R12,R6     ;number of word
                  MOV     R13,R7     ;points to the receive buffer; MGS modif!!!
                  ADD     R7,R6    
		  CLR	  data_r			; reset data_r
		  CLR	  wake_up_counter		; reset wake_up_counter
		  CLR	  RSTAT				; reset receive status register, RSTAT = 0, 
							; detecting the Trainingssequence

                  ;CALL   program_RX_Mode_1             ; new for TRF6901
		  BIC.B	  #tx,&P2OUT	                ; TXDATA(P1.4) is reset -> new for 6901
                  BIS.B   #l_h, &P3OUT                  ; set LEARN =HIGH, new for 6901
		  BIS.B	  #mode,&P3OUT			; Mode =1 -> receive FSK in learn mode
		  BIS.B	  #stdb_trf6901,&P3OUT		; TRF6900 active, STANDBY(P2.5) is high
		  CALL	  #wait_lockdet
		  BIS.B	  #01h,&IE1			; enable Watchdog Timer interrupt

		  MOV	  #TACLR+CONTUP+MCLK,&TACTL	; interrupt enable, clear Timer_A, 
		  ;MOV	  #CCIE+CAP+CMANY+SCS,&CCTL2	; interrupt enable, capture mode, both edges
		  MOV	  #CCIE+CAP+CMNEG+SCS,&CCTL2	; interrupt enable, capture mode, Negative edge
		  BIS	  #RF_preamble_wait,&_ComFlag 	; start the state machine
                  CLR     R13
      	          ;BIS.B	  #TEST_FLAG,&P1OUT							; mark the start of start bit
                  BIS     #ADC12SC,&ADC12CTL0;                 // Start conversion

loop_receive_training_seq
check_rstat2

		  BIT	  #LOCAL_BUTTON_PUSHED+RF_timedout_flag,&_ComFlag 	; UART receive char is valid
		  JNZ	  end_receive_RF_stop1
                  BIT     #1,&ADC12IFG
                  JZ      check_wake_up_counter
                  CALL    #HANDLE_RSSI
check_wake_up_counter
		  CMP	  #010h,wake_up_counter		; 16 equal pulses in succession 
		  ;CMP	  #08,wake_up_counter		; 8 equal cycles in succession 
                  JL	  loop_receive_training_seq	;  
;;;;;;;                   JMP	  loop_receive_training_seq	;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;***********************************
receive1	  MOV	  #WDTPW+WDTHOLD,&WDTCTL	; stop Watchdog Timer
		  BIC	  #022h,&TACTL			; stop Timer_A and disable interrupt
		  BIC	  #CCIE,&CCTL2			; disable interrupt
		  INCD	  R8				; RSTAT = 4

start_bit_reception					; waiting for the start_bit

                  BIC.B   #l_h,&P3OUT                   ;Go into hold mode New for 6901

start1		  MOV	  #TAIE+TACLR+CONTUP+MCLK,&TACTL  ; interrupt enable, clear Timer_A, continious 
							  ; up mode, MCLK as clock source
        	  MOV	  #SCS+CCIE+CAP+CMANY,&CCTL2	  ; interrupt enable, capture mode, both edges
                  CLR     R13

loop_start_bit
		  BIT	  #LOCAL_BUTTON_PUSHED+RF_timedout_flag,&_ComFlag 	; UART receive char is valid
		  JNZ	  end_receive_RF_stop2
		  CMP	  #04h,R8					; has the start bit been detected?	  
		  JEQ	  loop_start_bit				; wait for the start bit
		  JN	  end_receive_RF_invalid		        ; the received sequence is invalid
                  
;----------------------------- start bit detected ------------------------------------------------------
init_data_reception							; RSTAT = 6, Start Bit detected, Data Reception
  
		  BIS.B	  #TEST_FLAG,&P5OUT				; mark the start of start bit
         	  BIT.B	  #rx,&P2IN					; is RXDATA high or low?
		  RLC	  data_r					; push carry into the data register
		  BIC.B	  #TEST_FLAG,&P5OUT				; mark the start of start bit

                  CLR     R10                                           ;R10 =counter
		  MOV	  #01h,bits_r

init_rx_bit_counter

word_reception_loop
		  BIS	  #CPUOFF+GIE,SR				; go to sleep!
		  BIS.B	  #TEST_FLAG,&P5OUT				;mark the start of start bit
									; wake up from WDT (Timer Mode, every 26s)									   
		  BIT.B	  #rx,&P2IN					; is RXDATA high or low?

read_data
		  RLC	  data_r					; push carry into the data register
		  BIC.B	  #TEST_FLAG,&P5OUT				;mark the start of start bit
		  INC	  bits_r
		  CMP	  #010h,bits_r					; receive 16 bits in row
		  JNE	  word_reception_loop			        ; haven't received 8bits yet

store_data
		  INV	  data_r					; the received data is inverted!
		  MOV	  data_r,0(R7)				        ; store received data to RAM
                  INCD    R7
                  CMP     R6,R7
		  JEQ	  ready_to_end					; receive the next word
		  CLR	  bits_r
;******************************resynch occurs here
resynch_word_reception
		  BIC.B	  #01h,&IE1					; disable Watchdog Timer interrupt
		  MOV	  #TAIE+TACLR+CONTUP+MCLK,&TACTL		; interrupt enable, clear Timer_A, 
		  MOV	  #SCS+CCIE+CAP+CMPOS,&CCTL2			; int enable, capture mode,rising edge(neg start).Bits are inverted
wait_for_data
		  CMP	  #010h,bits_r					; receive 16 bits in row
		  JEQ	  store_data                                    ;if packet received, process
		  BIT	  #LOCAL_BUTTON_PUSHED+RF_timedout_flag,&_ComFlag 	;else, check UART
		  JNZ	  end_receive_RF
                  JMP     wait_for_data

ready_to_end
		  NOP
                  MOV     R8,R12
		  BIS	  #rf_R_buf_full,&_ComFlag			 ;2 stands for data received, 
more_xfer

end_receive_RF
		  BIC	  #CCIE+CCIFG,&CCTL0		; disable CCR0 interrupt
		  BIC	  #CCIE,&CCTL2			; disable CCR2 interrupt
		  BIC	  #022h,&TACTL			; stop Timer_A and disable interrupt	 
		  BIS	  #MC1,&TACTL			; Start TA Continous Mode
		  MOV	  #WDTPW+WDTHOLD,&WDTCTL	; stop Watchdog Timer
		  BIC.B	  #stdb_trf6901,&P3OUT		; clear STDBY(P2.5), TRF6900 in standby mode
wait_RF
                  BIC.B   #080h,&P3OUT                  ;turn off LED,ready to receive
		  RET

end_receive_RF_invalid                                  ;invalid start bit
                  MOV    R8,R12
                  ADD    #0200h,R12
                  JMP    end_receive_RF
end_receive_RF_stop1                                    ; manually stopped while waiting for pre-amble
                  MOV    R8,R12
		  ADD    #0100h,R12
		  JMP    end_receive_RF
end_receive_RF_stop2                                   ;manually stopped while waiting for start bit
                  MOV    R8,R12
                  ADD    #0300h,R12
                  JMP    end_receive_RF 		    

bad_lock_detect
                   MOV   #0200h,R12
                   JMP   end_receive_RF

;************************************ wait for lockdetect **********************************************
; last change: 3/31/03
;
; just wait  miminum 1 ms for the IC to settle down 
;
;*******************************************************************************************************

wait_lockdet
		  MOV	  #1000,wait_r			;retry about 10 times ;MGS was 01000, thus octal!

not_yet 
                  DEC	   wait_r			;tried enough?
		  JNZ	   not_yet		        ;if not try again,this calls for diagnostic
                  NOP      
                  RET                        


;*******hold radio for RSSI test

hold_radio
                  BIC.B   #l_h,&P3OUT                 ;New for 6901
		  BIS.B	  #stdb_trf6901,&P3OUT		; TRF6900 active, STANDBY(P2.5) is high
		  BIS.B	  #mode,&P3OUT			; receive FSK in learn mode
		  CALL	  #wait_lockdet
                  RET
;************release radio from RSSI test
release_radio
                  BIS.B   #l_h,&P3OUT                 ;New for 6901
		  BIC.B	  #stdb_trf6901,&P3OUT	      ; clear STDBY(P2.5), TRF6900 in standby mode

                  RET
HANDLE_RSSI
                  BIC.B   #RED,&P6OUT                 ; Decide which LED to light according to
		  CMP	  #ORN_TH,&ADC12MEM0	      ; compare the value of the counter with the 
                  JGE     next1                       ; start with red turned on
                  BIS.B   #YEL+ORN+GRN,&P6OUT
                  JMP      ret_rssi

next1             BIC.B   #RED+ORN,&P6OUT             ;RSSI Voltag
                  CMP     #YEL_TH,&ADC12MEM0          ;< 0x313 = < 0.300 Volts = < -75 dbm from RSSI curve
                  JGE     next2
                  BIS.B   #YEL+GRN,&P6OUT
                  JMP      ret_rssi

next2             BIC.B   #RED+ORN+YEL,&P6OUT         ;< 0x627 = < 0.600 Volts = < -55 dbm 
                  CMP     #GRN_TH,&ADC12MEM0          ;< 0xA41 = < 1.000 Volts = < -30 dbm
                  JGE     next3
                  BIS.B   #GRN,&P6OUT
                  JMP      ret_rssi
next3             BIC.B   #RED+ORN+YEL+GRN,&P6OUT     
ret_rssi
                 BIC     #1,&ADC12IFG
                 BIS     #ADC12SC,&ADC12CTL0;         ; Start conversion again
                 RET                 
                  
                  
                  

;	COMMON	INTVEC
    .pseg intvec1,abs=0xFFE0+PORT2_VECTOR
    DW	PORT2_INT
    .pseg intvec2,abs=0xFFE0+TIMERA1_VECTOR
	DW	Timer_A1
	DW	Timer_A0
   .pseg intvec3,abs=0xFFE0+WDT_VECTOR
	DW	WDT
	END
       