;***************************************************************
;*						Datawords
;* Values for the programming of the 4 data words of the TRF6901
;* 
;*
;*
;***************************************************************
;***********************************************************************************************************
; Timer_A settings
;***********************************************************************************************************

CONTUP		   EQU	   020h		   ; continuous up mode
MCLK		   EQU	   0200h	   ; MCLK as clock source
CMPOS		   EQU	   04000h	   ; rising edge
CMNEG		   EQU	   08000h	   ; falling edge
CMANY		   EQU	   0C000h	   ; any edge
CCIA		   EQU	   0000h	   ; input select A
CCIB		   EQU	   1000h	   ; input select B
GND		   EQU	   2000h	   ; input select GND
VCC		   EQU	   3000h	   ; input select VCC

;************************************************************************************************************
;*
;*  Portselection
;*
;*
;************************************************************************************************************
;***********************************************************************************************************
;port2
;***********************************************************************************************************
tx			   EQU	   04h		   ; P2.2 TXDATA transmit data to the TRF6901
rx			   EQU	   01h		   ; P2.0 RXDATA receive data from the TRF6901
;***************************************************
;Port 4
;***************************************************
strobe		   EQU	   02h		   ; P4.1 STROBE Programming end signal for the TRF6901 
data		   EQU	   01h		   ; P4.0 DATA Programming data for the TRF6901 
mode		   EQU	   080h		   ; P3.7 Mode 1 or Mode 0	 
clk		   EQU	   040h		   ; P3.6 CLK Programming clock for the TRF6901
stdb_trf6901       EQU	   020h		   ; P3.5 Low-Power-Mode for the TRF6901
l_h	           EQU	   010h		   ; P3.4 Lock detect, the PLL has locked at the selected frequency 

;***********************************************************************************************************
;port2
;***********************************************************************************************************

;LED3		   .equ	   08h		   ; P2.3 is now comparator  
spi_in		   EQU	   04h		   ; P2.2 SPI data in
eeprom		   EQU	   01h		   ; P2.0 eeprom select, active low


;************************************************************************************************************
;*
;* SYSTEM DEFININTIONS 
;*
;*
;************************************************************************************************************

WDT_250ms	   EQU	   05A1Dh	   ; Watchdog, timer, clear,ACLK,/8K
MSTOP		   EQU	   080h		   ; Stop Mode
MCONT		   EQU	   020h		   ; Continuous Mode


;Register:
#define   data_r R4
;data_r				         EQU	 R4		; the current byte, that is being received or transmitted
#define   word_trf  R4
#define   data_r0   R4

#define   bits_r    R5
#define   data_r1   R5 

#define   word_h		 R6				  ; the high "byte" of the TRF6900 programming word
#define   data_r2		 R6

#define   address_stop_r	 R7				  ; after the end of receive the address_r -> address_stop_r is used break-off 
#define   data_r3		 R7 			  ; info for the following transmition
#define   word_l		 R7				  ; the low "byte" of the TRF6900 programming word

												  ; (no collusion with res_new_r possible!)

#define   wake_up_counter	 R10			  ; counter for valid pulses during training phase
;#define   counter		 R10			  ; universal counter


#define   current        	 R11			  ; buffer for the current result of the Timer_A
#define   previous		 R12			  ; buffer for the previous result of the Timer_A

#define   res_old_r		 R13			  ; old value of the CCR2 Timer
#define   res_new_r		 R14			  ; new value of the CCR2 Timer difference gives width 
;							  ; -> res_r (both only during RF reception)

#define   res_r			 R15			  ; width of the right received puls	only during RF reception
#define   tr_counter		 R15			  ; counter for the training sequence
#define   rs232_counter		 R15			  ; used for the detection of the high and low byte during the rs232 
												  ; reception



#define data_rx_state		 0200h			  ; stores the reception state


;bits

#define   clock_neu_1		 070h			  ; ACLK 1/8 from external crystal
#define   clock_neu_2		 0C8h			  ; MCLK directly from LFXTCL
												  ; SMCLK directly from MCLK
