/**********************************TRF6901_demo.c************************
 *
 *    DESCRIPTION: TRF6901 Demonstration CODE
 *
 *    AUTHOR:  CnoWire
 *
 *    HISTORY: first version  3/14/03  
 *
 
 ********************************************************************************/

/** include files **/
#include <msp430x44x.h>
//#include "in430.h"
#include <stdio.h>
#include "f_map.h"


/** local definitions **/
//TRF6901 control signals
#define TRF_STROBE 0x2
#define TRF_DATA 1
#define TRF_MODE 0x80
#define TRF_CLK 0x40
#define TRF_STANDBY 0x20
#define TRF_LH 0x10
#define tx  0x4		      //; P2.2 TXDATA transmit data to the TRF6901
#define rx  0x8 	      //; P1.3 RXDATA receive data from the TRF6901
#define FALSE 0

#define LED_YELLOW 0x80
#define LED_GREEN 0x40
#define LED_ORANGE 0x20
#define LED_RED 0x10
#define LEDALL 0xf0
#define FSEL4 1
#define FSEL3 0x80
#define FSEL2 0x40
#define FSEL1 0x8
#define NONE 0
#define RED  1
#define ORANGE 2
#define GREEN  4
#define YELLOW 8
#define MAXWORD 3                              //size in word 
#define MAXBYTE MAXWORD*2                      //size in byte
#define FALSE 0
#define TRUE 1
#define ACK_CODE 0x0606
#define COM_SUCCESSFUL 6
#define LOCAL_BUTTON_PUSHED 0x8               //equivalent to uart_R_buf_full

#define rf_X_buf_full	0x4                   //rf xmit buffer has new data
#define RF_timedout_flag 0x80             
#define rf_R_buf_full	0x10
#define F_change_flag 0x100
#define MAX_RETRAN 3

#define SEC5 50                               /* default settings */
#define SEC10 100
#define RESPONSE_TIMEOUT SEC10
#define SAMPLEPERIOD 0x7800;
#define XTAL_OFFSET 0
#define DEBOUNCE     7690                    //25ms    
#define US 0x40
#define EU 0

/** external functions **/

extern void Set_Clk(void);
extern void Set_DCO(int);
extern void program_TRF69(unsigned int, unsigned int);  //trf6901.s43
extern int receive_RF(unsigned char, unsigned int *);
extern void send_RF(unsigned int,unsigned int*);
/** external data **/

/** internal functions **/

/** public data **/

int f_sel,fsel_update; 
int ComFlag; 

/** private data **/

/** public functions **/
void InitIo(void);
int  GetFsel(void);


void configure_trf6901(void);
void program_TX_Mode_0(void);           //		 
void program_RX_Mode_1(void);           //
void program_WORD_C(void);              //		   
void program_WORD_D(unsigned long);     //
void program_Tx_C(void);
void program_Rx_C(void);

void InitButton(void);
void ClearLocalButton(void);
void ClearRemoteButton(void);
void ParsePacket(void);
unsigned char MonitorRemoteTransmitter(void);
int CheckChecksum(unsigned char);
void ParsePacket(void);
void LEDQueBlink(void);
void GetXmitPacket(void);
void ChecksumGen(unsigned char);
char SendRFPacket(void);
void InitCOMTimer(int );
void DisableButtonInt(void);
void TBIntEnable(int,int);
void InitADC(void);


/** private functions **/

struct trf69_control{
   unsigned int a_word_l, a_word_h, b_word_l, b_word_h;
   unsigned int c_word_l, c_word_h, d_word_l, d_word_h;
 }trf6901;  
enum OpState{
	RECEIVER_NORMAL,
	LED_REMOTE_QUE,
	LED_RESPONSE_RESULT,
	WAIT_FOR_LOCAL_BUTTON_RESPONSE,
	TRANSMIT_BUTTON_RESPONSE,
        RECEIVER_WAITING_FOR_RESPONSE,
	LOCAL_BUTTON_ACTIVE,
	TRANSMIT_LOCAL_BUTTON_PACKET,
	BUTTON_ACTIVE
};
enum OpState current_state, past_state;

struct BUTTON_ID{
 char color[4];
 char cnt;
}local_button,remote_button;

struct RF_XMIT_PACKET {
  int packetsize;
  unsigned int xmit[MAXWORD];	 
  unsigned int rcv[MAXWORD];   
}buf;   
unsigned char REGION;
unsigned char retry_count;
unsigned char failed;
int ms100_cnt;                  //5 sec delay
unsigned int Btime;             
unsigned int Btime1_5;


void main(void)
{
   int i;
   unsigned char temp;
 
   WDTCTL = WDTPW+WDTHOLD;// Stop watchdog timer
   InitIo();
   InitADC();
   Btime=64;
   REGION=P6IN&US;            //check the region of operation
   Btime1_5=60;               //new 4 mhz xtal timing
   f_sel = GetFsel();
   for(i=0;i<10;i++)          //retry multiple times to make sure the RF chip programmed during power up
     configure_trf6901();
   buf.packetsize = MAXBYTE;
   current_state = RECEIVER_NORMAL;  
   while(1){
     if(current_state==RECEIVER_NORMAL){
       fsel_update =GetFsel();             //get new frequency select, only in idle state    
       if(fsel_update != f_sel){
         f_sel = fsel_update;
         configure_trf6901();
      }
     }
     switch(current_state){
      case RECEIVER_NORMAL:              
        InitButton();                              //get ready to accept button or RF recieve
        past_state=current_state;
        ComFlag&=~(LOCAL_BUTTON_PUSHED + RF_timedout_flag);
        temp=MonitorRemoteTransmitter();            //park on receive
        if((ComFlag & LOCAL_BUTTON_PUSHED)){        //local user initiate button sequence
          current_state = LOCAL_BUTTON_ACTIVE;
        }
        else if(temp==0){                           //remote user sent LED que
          current_state = LED_REMOTE_QUE;
        }
 
        else{ 
          current_state=RECEIVER_NORMAL;              //error during receive, just go back
        }

       break;
      case LOCAL_BUTTON_ACTIVE:                       //local button is pressed
         while((local_button.cnt<4)&&(ms100_cnt!=0)); //wait until 4 pushes or timed out
         TBCCTL1&=0xffed;                             // turn off further INT
         DisableButtonInt();
         for(i=0;i<4-local_button.cnt;i++){          //back fill blank
           local_button.color[local_button.cnt+i]=NONE;
         }
         past_state=current_state;
         current_state=TRANSMIT_LOCAL_BUTTON_PACKET;
        break;
      case LED_REMOTE_QUE:                            //received a sequence of LED from remote display  
         LEDQueBlink();
         past_state=current_state;
         current_state = WAIT_FOR_LOCAL_BUTTON_RESPONSE;
        break;
      case WAIT_FOR_LOCAL_BUTTON_RESPONSE:
         past_state=current_state;
         ClearLocalButton();
         P1IFG&=0xf0;                                 //clear all interrupt
         P1IE|= 0xf;                                  //enable button interrupt
         ms100_cnt=30;                                //3 second timeout
         TBIntEnable(1,0x7800);
         while((local_button.cnt<4)&&(ms100_cnt!=0));
         TBCCTL1&=0xffed;                             // turn off further INT
         DisableButtonInt();                          //no more interrupt for now
         for(i=0;i<4-local_button.cnt;i++){
           local_button.color[local_button.cnt+i]=NONE;
         }
         current_state=TRANSMIT_LOCAL_BUTTON_PACKET;
        break;  

      case TRANSMIT_LOCAL_BUTTON_PACKET:
          GetXmitPacket();                            //format for xmit packet      
          retry_count=0;                              //set up for built it retry
          while(retry_count<2){
            failed=SendRFPacket();                    //send packet with ACK
            if(failed==MAX_RETRAN){
              retry_count+=1;                           //try to recover one more time.   
           }
           else
             retry_count=COM_SUCCESSFUL; 
           }
         
          if(past_state==WAIT_FOR_LOCAL_BUTTON_RESPONSE){  //if response button sequence was sent,this completes the loop
              past_state=current_state;
              current_state=RECEIVER_NORMAL;
           }   
          else if (past_state==LOCAL_BUTTON_ACTIVE){      //local initiated and sent the que
              past_state = current_state;
              ClearRemoteButton(); 
              current_state = RECEIVER_WAITING_FOR_RESPONSE; 
          }    
 	break;
      case RECEIVER_WAITING_FOR_RESPONSE:
           ms100_cnt=RESPONSE_TIMEOUT;                    //5 second timeout
           TBIntEnable(1,0x7800);                         //0x7800 ~ 100mS
           ComFlag&=~(LOCAL_BUTTON_PUSHED + RF_timedout_flag);
           do{
             temp=MonitorRemoteTransmitter();             //either timeout or good COM
           }while((ms100_cnt>0)&&(temp!=0));

           TBCCTL1&=0xffed;                               //clear timer activity
           current_state = LED_RESPONSE_RESULT;
        break;

      case LED_RESPONSE_RESULT:
           if(ms100_cnt<=0){                              //if timed out,then error
             P1OUT|=LED_RED;
           }
           else{
             remote_button.cnt=0;
             for(i=0;i<4;i++){
              if(remote_button.color[i]!=local_button.color[i]){
                P1OUT|=LED_ORANGE;                        //if error in response
                remote_button.cnt+=1;                     //accumulate error, using this as the counter
              }
             }    
             if(remote_button.cnt==0)P1OUT|=LED_GREEN;      //any mismatch is error
           }
           ms100_cnt= 10;                                   //blink for 1 sec
           TBIntEnable(1,0x7800);
           while(ms100_cnt>0);                              //either timeout or good COM
           P1OUT&=~(LED_RED+LED_GREEN+LED_ORANGE);
           past_state=current_state;
           current_state=RECEIVER_NORMAL;
           
          //Blink Green for good
          //blink Red for bad
         break;
  }  
 }//while 1

}//end main

void InitIo(void)
{
  int i;

  P6OUT|=0xf;             //All RSSI led off 
  P6DIR |= 0xf;           //RSSI LED driver configured as ouput
  P6SEL|=0x80;            //RSSI is AD function

  P4DIR|= TRF_STROBE + TRF_DATA;        //P4.0, P4.1 are output
  P3DIR|= TRF_CLK  + TRF_STANDBY + TRF_MODE + TRF_LH;  //P3 OUTPUT

  P2DIR|= tx;

  P1DIR|= LEDALL;
  P1OUT&= ~LEDALL;
  P1IES |= 0xf;                 //all int on high to low trans
  P2SEL=0x33;                   //select UART, select TimerA capture
  Set_DCO(30000);               //set dco for 32Khz clock   
  //initialize UART port. Default Baud =38.4K
  UTCTL0=0x20;
  URCTL0=0x8;
  UBR00=0x40;		    
  UBR10=0;
  UMCTL0=0x0;
  UMCTL0=0;
  UCTL0=0x11;
  UCTL0&=0xfe;
  ME1|=URXE0;       //enable receiver
  //blink All LED to indicate power up condition
  for(i=0;i<30000;i++){
   P1OUT |= LEDALL;
   P6OUT&=0xf0;     
  }
  P1OUT&=~(LEDALL);
  for(i=0;i<30000;i++);       //arbitrary wait until TRF6901 warms up

  P6OUT|=0xf;
  P5DIR|=1;           //use this for test flag
  P5OUT&=0xfe;        //start off low
}

void configure_trf6901(void)
{
    unsigned long long_reg;
    P4OUT&=0xfe;                   //start with 0 on data
    program_TX_Mode_0();           //		 
    program_RX_Mode_1();           //
    program_WORD_C();              //		   
    long_reg = (unsigned long)XTAL_OFFSET;
    long_reg<<=16;
    long_reg|= TRF69_Dword;
    program_WORD_D(long_reg);
}




/**************************** Programming the TRF6900 **************************************************														   
                                program_TX_Mode_0
; purpose: programs the Register A  for FSK transmission(changed from original TI config)
;			(Mode 0, A-word)
;
;*******************************************************************************************************
*/
void program_TX_Mode_0(void)
{
    unsigned long trf69;
    int pointer;
     pointer=(int)f_sel;
     trf69 = TRegA[pointer];
     trf6901.a_word_h = (unsigned int)(trf69>>16);
     trf6901.a_word_l = ((unsigned int)trf69);
     program_TRF69(trf6901.a_word_h,trf6901.a_word_l);
}



/*                               program_RX_mode_1                                                     */
                                 /* used to be */
/******************************* program_DDS1_send *****************************************************
; purpose: programs B register for reception (see further settings below)
;			(Mode 1, B-word)
;
;*******************************************************************************************************/


void program_RX_Mode_1(void)
{
    unsigned long trf69;
    int pointer;
     pointer=(int)f_sel;
     trf69 = RRegB[pointer];
     trf6901.b_word_h = (unsigned int)(trf69>>16);
     trf6901.b_word_l = ((unsigned int)trf69);
     program_TRF69(trf6901.b_word_h,trf6901.b_word_l);
}


/*******************************program_WORD_C*********************************************************/
/* purpose: programs word C (see further settings below)                                             */

void program_WORD_C(void)
{
   unsigned long trf6901_word;
     trf6901_word = TRF69_Cword;
     trf6901.c_word_h = trf6901_word>>16;
     trf6901.c_word_l = trf6901_word;
     program_TRF69(trf6901.c_word_h,trf6901.c_word_l);
}




/*******************************program_WORD_D*********************************************************/
/*******************************************************************************************************/

void program_WORD_D(unsigned long trf69)
{
    
     trf6901.d_word_h = (unsigned int)(trf69>>16);
     trf6901.d_word_l = (unsigned int)trf69;
     program_TRF69(trf6901.d_word_h,trf6901.d_word_l);
}




int GetFsel(void)
{
  int in0, in1,in2;
  in0 = P3IN&FSEL4;
  in0<<=3;
  in1 = P2IN & (FSEL3+FSEL2+FSEL1);   // Get remaining select
  in0|=(in1>>5);                      // align FSEL3 and FSEL2
  in1 = P2IN & FSEL1;                 // align FSEL1
  in0 |= in1>>3;
  in0=~in0;                           //reverse polarity 
  in1=in0 & 0xf;
  in0 = (in1>>3)&0x1;
  in2 = (in1>>1)&0x2;
  in0 |=in2;
  in2 = (in1<<1)&0x4;
  in0 |= in2;
  in2 = (in1<<3)&0x8;
  in0 = in0 |in2;
  return in0;
}

/* init_button()
  clear all button state
  initialize button interrupt 
  Initialize the timerB

*/

void InitButton(void)
{
  ClearLocalButton();
  ClearRemoteButton();
  P1IFG&=0xf0;           //clear all interrupt
  P1IE|= 0xf;           //enable button interrupt
  

}

void ClearLocalButton(void)
{
  local_button.color[0] = NONE;
  local_button.color[1] = NONE;
  local_button.color[2] = NONE;
  local_button.color[3] = NONE;
  local_button.cnt = NONE;
}

void ClearRemoteButton(void)
{
  remote_button.color[0] = NONE;
  remote_button.color[1] = NONE;
  remote_button.color[2] = NONE;
  remote_button.color[3] = NONE;
  remote_button.cnt = NONE;
}

unsigned int T1;

interrupt [PORT1_VECTOR] void Port1INT(void)
{
  unsigned char intflag;

  P1IE &=0xf0;                 //turn off any int
  intflag=P1IFG;
  if((intflag & RED)==RED){
    local_button.color[local_button.cnt]=RED;
  }
  else if((intflag & ORANGE)==ORANGE){
     local_button.color[local_button.cnt]=ORANGE;
  }   
  else if((intflag & GREEN)==GREEN){
    local_button.color[local_button.cnt]= GREEN;
  }

  else if((intflag & YELLOW)==YELLOW){
    local_button.color[local_button.cnt]= YELLOW;
  }

  if(intflag & 0xf){
    P6OUT|=0xF;                                 //clear RSSI
    ComFlag|=LOCAL_BUTTON_PUSHED;
    local_button.cnt++;
    if(local_button.cnt<4){
      ms100_cnt=30;                    //reset timer to 3 second
      TBIntEnable(1,0x7800);
      T1=TBR;
      while(TBR<(T1+DEBOUNCE));      //wait until all the transition is done
      P1IFG&=0xf0;                 //clear all Int flag
      P1IE |=0xf;                 //enable further int
    } 
    
  } 

}

int error1;

unsigned char MonitorRemoteTransmitter(void)
{
  ComFlag&=~rf_R_buf_full;
  error1 = receive_RF(buf.packetsize,buf.rcv);	
  TBCCTL2&=0xffed;              //packet received,disable further timer activity
  if(((ComFlag&LOCAL_BUTTON_PUSHED)==0)&&((error1&0x300)==0)){	   //set flag means got data
     if(CheckChecksum(buf.packetsize)==0){ //check checksum
         buf.xmit[1]=ACK_CODE;  
         buf.xmit[0]=ACK_CODE;
	 ComFlag|=rf_X_buf_full;        //enable transmitter
	 send_RF(4,buf.xmit);	        //send ACK word
         ParsePacket();                 //rx_buff has the data
	 return 0;                      //good data packet
       }//good checksum
     }//received data   
  return 1;                             //failed to get new data
}

void ParsePacket(void)
{
  remote_button.cnt=0;
  remote_button.color[0] = buf.rcv[0] ;
  remote_button.color[1] = buf.rcv[1]>>8;
  remote_button.color[2] = buf.rcv[1] ;
  remote_button.color[3] = buf.rcv[2]>>8;
  remote_button.cnt=4;
}


/************************************check_checksum****
;  1. buf.rcv has the data. checksum is located at LSB of buf.rcv[maxword]
;  4. sets flag to 080h if the checksum error.
;  5. 
;*****************************************************/

int CheckChecksum(unsigned char num)
{
  int sum,k;
  unsigned int var;
  unsigned char var_byte;
  num = num>>1;//convert to word
  num-=1;
  sum=0;
  for(k=0;k<num;k++){
   var=buf.rcv[k];
   var_byte=var;
   sum+=var_byte;
   var_byte=(var>>8)&0xff;
   sum+=var_byte; 
  }
 var=buf.rcv[k];
 var_byte=(var>>8)&0xff;
 sum+=var_byte;
 var_byte=(unsigned char)sum&0xff;
 if(var_byte==((unsigned char)(var&0xff)))
   return 0;
 else
  return 1;
}



void LEDQueBlink(void)
{
   int i;  
   for(i=0;i<remote_button.cnt;i++){
     if(remote_button.color[i]!=NONE){
       P1OUT |= (remote_button.color[i]<<4);
       //ms100_cnt= 10;                     //blink for .5 sec
       ms100_cnt= 5;                     //blink for .5 sec
       TBIntEnable(1,0x7800);             //set for 1 second blink
       while(ms100_cnt>0);                //wait until done
       P1OUT&=~(remote_button.color[i]<<4);
       ms100_cnt= 3;                     //if same color, more off time
       TBIntEnable(1,0x7800);             //set for 1 second blink
       while(ms100_cnt>0);                //wait until done
     }//if
   }//for     
}
 
interrupt [TIMERB0_VECTOR] void TimerB0(void)
{

}
/**********************************************************************
  Timer_B1 LED and push button timer
 **********************************************************************/
interrupt [TIMERB1_VECTOR] void Timer_B(void)
{
 switch( TBIV )
 {
   case  2:                     /* CCR1 */
      TBCCR1=(TBR + 0x7800) & 0xffff;   //turn on the interrupt
      if(ms100_cnt--==0){
           TBCCTL1&=0xffed;     // timed out turn off further INT
           ComFlag|=RF_timedout_flag;  //double function as RF timeout
      }
      break;
   case  4:                     
      TBCCTL2&=0xffed;            //round trip ACK timer. If ACK is not received within this time, ACK
      ComFlag|=RF_timedout_flag;  // monitor is terminated.
      //P4OUT&=~stdb_trf6900;	 //disable the chip to get rid of noise
     break;
   case  6:                      /* CCR3-Sampling period timer */
      break;
 }
}
/*******************************************************************************
  TimerB clock = 2.4576Mhz/8 = 307200
******************************************************************************/


void TBIntEnable(int num, int time)
{
  if((TBCTL&0x30)==0){            //if the counter is halted
    TBCTL=0x2C4;                //stop counter
    TBR=0;                      //clear counter
    TBCTL=0x2E0;               //continuous up
   }
  switch(num){
   case 1:
                                      //ms100_cnt=10;
     TBCCR1=(TBR + time) & 0xffff;   //turn on the interrupt
     TBCCTL1 =0x4010;
    break;
   case 2:
      TBCCR2=(TBR+ time)&0xffff;     // ACK response time out of 1 /32 sec=0xff
      TBCCTL2=0x4010;            //ACK response timeout          
    break;
  }       
}
void GetXmitPacket(void)
{
  buf.xmit[2]=0xfe00;
  buf.xmit[2]|=local_button.color[0];
  buf.xmit[1]=local_button.color[1];     
  buf.xmit[1]<<=8;     
  buf.xmit[1]|= local_button.color[2];
  buf.xmit[0]= local_button.color[3];
  buf.xmit[0]<<=8;
  ChecksumGen((unsigned char)buf.packetsize);
}
/*********************Checksum_gen**********************************************
;for sending data through RF channel only
;The calling function must load counter with the data packet size in bytes
;******************************************************************************/
void ChecksumGen(unsigned char num)
{
   unsigned int sum;
   sum=0;       //initialize paremeters
   num=num>>1;
   num=num-1;
   while(num>0){
     sum+=buf.xmit[num]>>8;
     sum+=(buf.xmit[num]&0xff);
     num--;
   }
     sum+=(buf.xmit[0]>>8);
     sum&=0xff;
     buf.xmit[0]+=sum;
     return;
}


char SendRFPacket(void)
{
  static char re_cnt;
  static int errorcode;
  static char attempt;

  re_cnt=0;
  attempt = (char)MAX_RETRAN;
  while(re_cnt<attempt){                     //the retry here is 3 to ensure getting through


     InitCOMTimer(buf.packetsize);            //Set ACK timeout

     ComFlag|=rf_X_buf_full;                 //indicate rf xmit buff full

     send_RF(buf.packetsize,buf.xmit);	

     ComFlag&=~(LOCAL_BUTTON_PUSHED + RF_timedout_flag+rf_R_buf_full); //turn off all flag that will disrupt rcv

     errorcode = receive_RF(4,buf.rcv);   //receive rf data routine

     if((errorcode & 0x300)==0){         

        TBCCTL2&=0xffed;              //packet received,disable further timer activity
        if((buf.rcv[0]==ACK_CODE)){
          return attempt+1;                   //finished, bail out
        }
        else re_cnt++;
     }//received ACK
     else re_cnt++;              //else timed out
  }//end of while back to checking . 3 tries

 return re_cnt;

}

/*
   Time out after the round trip
  1. Compute xmit time  based on packet size and baudrate.
  2. Time=(1/Baud_rate)*(PREAMBLE_LENGTH + data packet size+ PREAMBLE_LENGTH+data_packet)
  3. Time= (1/Baud_rate)*(100 cylces+Packet_SIZe+ 100 cycles + 32) in useconds;
  4. This include time to xmit and time to receive ACK 
  5. Use TimerB2.
  6 Timer B2 is SMCLK/8 = 2.4576Mhz/8 = 307200 hz= 3.25us
*/
void InitCOMTimer(int size)
{
  int tbccr2;
  int BitCount;
  BitCount= (size+4)*8;
  BitCount+=1000;                             //roundtrip preamble+ overhead time + buffer 
  tbccr2= (int)((BitCount*26.0)/3.25);       //these numbers may need adjustment 
  if((TBCTL&0x30)==0){
     TBCTL=0x2C4;                //stop counter
     TBR=0;                      //clear counter
     TBCCR2=tbccr2;              //loop time
     TBCTL=0x2E4;       //continuous up
  }
 else
{
   tbccr2+=TBR;         //timer already moving 
   if((TBCTL&0x1800)==0x800)tbccr2&=0xfff;  //mask msb bit for various length counter
   else if((TBCTL&0x1800)==0x1000)tbccr2&=0x3ff;
   else if((TBCTL&0x1800)==0x1800)tbccr2&=0xff; 
   TBCCR2=tbccr2;              //loop time
 }                              //just count the offset
  ComFlag&=~RF_timedout_flag;
  TBCCTL2=0x4010;               //compare,CCIE,CAP
}

void DisableButtonInt(void)
{
  P1IE &=0xf0;

}

void InitADC(void)
{
  unsigned int i;  

  ADC12CTL0 = ADC12ON+REFON+SHT0_6;     // Setup ADC12, ref., sampling time
  ADC12CTL1 = SHP;                      // Use sampling timer
  ADC12MCTL0 = INCH_7+SREF_1;           // Select channel A7, Vref+

  for (i=0; i<0x3600; i++)              // Delay for reference start-up
  {                                     // this is required to ensure that
  }                                     // the reference is stable for a conversion
  ADC12CTL0 |= ENC;                     // Enable conversion

}        